pub enum GridError {
OutOfBounds(Coord),
AlreadyVisited(Coord),
}Variants
OutOfBounds(Coord)
The coordinate has no cell associated with it, as it’s out of the grid bounds.
AlreadyVisited(Coord)
The coordinate has previously been mutably borrowed from the iterator, and doing so again would break safety guarantees.